/*
**************************************************************************
                                 description
                             --------------------
    copyright            : (C) 2002 by Luis Carvalho
    email                : lpassos@mail.telepac.pt
**************************************************************************

**************************************************************************
*                                                                        *
*  This program is free software; you can redistribute it and/or modify  *
*  it under the terms of the GNU General Public License as published by  *
*  the Free Software Foundation; either version 2 of the License, or     *
*  (at your option) any later version.                                   *
*                                                                        *
**************************************************************************/


#include "pmlibrarybrowser.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qsplitter.h>
#include <qtimer.h>

#include <kurl.h>
#include <klocale.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <kdialog.h>
#include <kdiroperator.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kguiitem.h>

#include <stdlib.h>

#include "pmlineedits.h"
#include "pmdialogeditbase.h"
#include "pmlibrarymanager.h"
#include "pmlibraryhandle.h"
#include "pmlibraryobject.h"
#include "pmlibraryiconview.h"
#include "pmlibraryobjectpreview.h"

// ************** PMLibraryBrowser **************************

PMLibraryBrowserView::PMLibraryBrowserView( PMPart* part, QWidget* parent /*= 0*/,
                                    const char* name /*=0*/ )
      : PMViewBase( parent, name )
{
   QHBoxLayout* hl = new QHBoxLayout( this );
   PMLibraryBrowserViewWidget* tv = new PMLibraryBrowserViewWidget( this );
   hl->addWidget( tv );
}

QString PMLibraryBrowserView::description( ) const
{
   return i18n( "Library View" );
}

PMLibraryBrowserViewWidget::PMLibraryBrowserViewWidget( QWidget* parent, const char* name )
      : QWidget( parent, name )
{
   setCaption( i18n( "Library Objects" ) );

   QVBoxLayout* vl = new QVBoxLayout( this, KDialog::spacingHint( ) );

   QHBoxLayout* hl = new QHBoxLayout( vl );

   m_pUpButton = new KPushButton( this );
   m_pUpButton->setGuiItem( KGuiItem( "", "up", "Up" ) );
   m_pNewSubLibraryButton = new KPushButton( this );
   m_pNewSubLibraryButton->setGuiItem( KGuiItem( "", "folder_new", "New Sub-Library" ) );
   m_pNewObjectButton = new KPushButton( this );
   m_pNewObjectButton->setGuiItem( KGuiItem( "", "filenew", "New Object" ) );
   m_pEditObjectButton = new KPushButton( this );
   m_pEditObjectButton->setGuiItem( KGuiItem( "", "fileopen", "Modify Object" ) );
   m_pDeleteObjectButton = new KPushButton( this );
   m_pDeleteObjectButton->setGuiItem( KGuiItem( "", "editdelete", "Delete Object" ) );
   QLabel* lbl = new QLabel( i18n( "Library: " ), this );
   m_pLibraryComboBox = new QComboBox( this );
   m_pLibraryComboBox->insertStringList( PMLibraryManager::theManager( )->availableLibraries( ) );
   m_pLibraryComboBox->setDuplicatesEnabled( false );
   m_pLibraryComboBox->setCurrentItem( 0 );
   hl->addWidget( m_pUpButton );
   hl->addWidget( m_pNewSubLibraryButton );
   hl->addWidget( m_pNewObjectButton );
   hl->addWidget( m_pEditObjectButton );
   hl->addWidget( m_pDeleteObjectButton );
   hl->addWidget( lbl );
   hl->addWidget( m_pLibraryComboBox );
   hl->addStretch( 1 );

   QSplitter *splitv = new QSplitter( this );
   m_pLibraryIconView = new PMLibraryIconView( splitv );
   m_pLibraryIconView->setMinimumSize( PMDialogEditBase::previewSize( )+20, PMDialogEditBase::previewSize( ) );
   m_pLibraryObjectPreview = new PMLibraryObjectPreview( splitv );
   vl->addWidget( splitv, 99 );

   // Connect all the objects
   connect( m_pUpButton, SIGNAL( clicked( ) ), SLOT( slotUpButtonClicked( ) ) );
   connect( m_pNewSubLibraryButton, SIGNAL( clicked( ) ), SLOT( slotNewSubLibraryClicked( ) ) );
   connect( m_pNewObjectButton,  SIGNAL( clicked( ) ), SLOT( slotNewObjectClicked( ) ) );
   connect( m_pEditObjectButton, SIGNAL( clicked( ) ), SLOT( slotEditClicked( ) ) );
   connect( m_pLibraryComboBox, SIGNAL( highlighted( const QString& ) ),
                                SLOT( slotPathSelected( const QString& ) ) );
   connect( m_pLibraryIconView, SIGNAL( selectionChanged( QIconViewItem* ) ),
                                SLOT( slotSelectionChanged( QIconViewItem* ) ) );

   // Set the selected library
   slotPathSelected( m_pLibraryComboBox->currentText( ) );
}

void PMLibraryBrowserViewWidget::resizeEvent( QResizeEvent* ev )
{
}

void PMLibraryBrowserViewWidget::slotPathSelected( const QString& str )
{
   PMLibraryHandle* h = PMLibraryManager::theManager( )->getLibraryHandle( str );
   if( h )
   {
      m_pLibraryIconView->setLibrary( h );
      m_pCurrentLibrary = h;
      // This can never be a sub library
      m_pUpButton->setEnabled( false );
   }
}

void PMLibraryBrowserViewWidget::slotIconViewRefresh( )
{
   m_pLibraryIconView->refresh( );
}

void PMLibraryBrowserViewWidget::slotIconViewSetLibrary( )
{
   m_pLibraryIconView->setLibrary( m_pFutureLibrary );
   m_pCurrentLibrary = m_pFutureLibrary;
   m_pFutureLibrary = NULL;
}

void PMLibraryBrowserViewWidget::slotSelectionChanged( QIconViewItem* item )
{
   PMLibraryIconViewItem* sel = static_cast<PMLibraryIconViewItem*>( item );

   if( sel->isSubLibrary( ) )
   {
      // It's a sub library
      m_pFutureLibrary = new PMLibraryHandle( sel->path( ) );

      QTimer::singleShot( 100, this, SLOT( slotIconViewSetLibrary( ) ) );
      m_pUpButton->setEnabled( true );
   } else
      // It's an object
      m_pLibraryObjectPreview->showPreview( sel->path( ) );
}

void PMLibraryBrowserViewWidget::slotUpButtonClicked( )
{
  QDir pathManipulator( m_pCurrentLibrary->path( ) );

  pathManipulator.cdUp( );
  m_pFutureLibrary = new PMLibraryHandle( pathManipulator.path( ) );
  if( !m_pFutureLibrary->isSubLibrary( ) )
     m_pUpButton->setEnabled( false );
  QTimer::singleShot( 100, this, SLOT( slotIconViewSetLibrary( ) ) );

  // Release the current Library
  delete m_pCurrentLibrary;
}

void PMLibraryBrowserViewWidget::slotEditClicked( )
{
   m_pLibraryObjectPreview->allowEdit( true );
}

void PMLibraryBrowserViewWidget::slotNewObjectClicked( )
{
   m_pLibraryObjectPreview->saveIfNeeded( );

   switch( m_pCurrentLibrary->createNewObject( ) )
   {
      case PMLibraryHandle::Ok:
         QTimer::singleShot( 100, this, SLOT( slotIconViewRefresh( ) ) );
         break;
      case PMLibraryHandle::ReadOnlyLib:
         KMessageBox::error( this, i18n( "This library is read only." ), i18n( "Error" ) );
         break;
      default:
         KMessageBox::error( this, i18n( "Could not create a new object." ), i18n( "Error" ) );
   }
}

void PMLibraryBrowserViewWidget::slotNewSubLibraryClicked( )
{
   m_pLibraryObjectPreview->saveIfNeeded( );

   switch( m_pCurrentLibrary->createNewSubLibrary( ) )
   {
      case PMLibraryHandle::Ok:
         m_pLibraryIconView->refresh( );
         break;
      case PMLibraryHandle::ReadOnlyLib:
         KMessageBox::error( this, i18n( "This library is read only." ), i18n( "Error" ) );
         break;
      default:
         KMessageBox::error( this, i18n( "Could not create a new sub library." ), i18n( "Error" ) );
   }
}

QString PMLibraryBrowserViewFactory::description( ) const
{
   return i18n( "Library View" );
}

#include "pmlibrarybrowser.moc"
